﻿using System.Net;
using System.Text.Json;
using System.Text;
using System.Device.I2c;
using System.IO.Ports;
using System;

namespace PremierVkrzAPI
{
    class Program
    {
        static float P, TPressure;
        static float TGyro, X, Y, Z, XX, YY, ZZ;
        
        static async Task Main(string[] args)
        {
            // Set the I2C bus ID and device address
            const int busId = 3; // I2C bus
            const int Pressure = 0x5D; // Device address on the I2C bus
            const int AccGyro = 0x6B;  // Device address on the I2C bus

            // Create an I2C devices
            using var devicePressure = I2cDevice.Create(new I2cConnectionSettings(busId, Pressure));
            using var deviceAccGyro = I2cDevice.Create(new I2cConnectionSettings(busId, AccGyro));

            // Init Gyro device
            InitGyro(deviceAccGyro);

            var listener = new HttpListener();
            //listener.Prefixes.Add("http://10.10.22.111:5000/");
            listener.Prefixes.Add("http://127.0.0.1:5000/");
            listener.Start();

            Console.WriteLine("Listening for HTTP POST requests...");

            while (true)
            {
                string requestBody;
                var context = await listener.GetContextAsync();

                if (context.Request.HttpMethod == "POST")
                {
                    using (var reader = new StreamReader(context.Request.InputStream, context.Request.ContentEncoding))
                    {
                        requestBody = await reader.ReadToEndAsync();
                    }



                    //Console.WriteLine(requestBody);

                    // Set CORS headers for preflight request
                    context.Response.Headers.Add("Access-Control-Allow-Origin", "*");
                    context.Response.Headers.Add("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
                    context.Response.Headers.Add("Access-Control-Allow-Headers", "Content-Type, Authorization");

                    // Respond to the preflight request with status code 200 No Content
                    context.Response.StatusCode = (int)HttpStatusCode.OK;

                    //context.Request.HttpMethod

                    ReadPressureTemp(devicePressure);
                    ReadAccGyroTemp(deviceAccGyro);
                    byte[] responseBytes = Encoding.UTF8.GetBytes("{\"TGyro\": " + TGyro + ",\r\n\"X\": " + X + ", \r\n\"Y\": " + Y + ", \r\n\"Z\": " + Z + 
                            ", \r\n\"XX\": " + XX + ", \r\n\"YY\": " + YY + ", \r\n\"ZZ\": " + ZZ + ", \r\n\"P\": " + P + ", \r\n\"TPressure\": " + TPressure + "}");

                    context.Response.ContentLength64 = responseBytes.Length;
                    await context.Response.OutputStream.WriteAsync(responseBytes, 0, responseBytes.Length);
                    context.Response.Close();



                }
                else // on OPTIONS
                {
                    context.Response.Headers.Add("Access-Control-Allow-Origin", "*");
                    context.Response.Headers.Add("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
                    context.Response.Headers.Add("Access-Control-Allow-Headers", "Content-Type, Authorization");
                    byte[] responseBytes = Array.Empty<byte>();
                    await context.Response.OutputStream.WriteAsync(responseBytes, 0, 0);
                    context.Response.Close();
                }

            }
        }
        static int Parse24(Span<byte> data)
        {
            if (data.Length < 3)
                throw new ArgumentException("Data span is too short to contain a signed 24-bit integer.");

            // Combine the bytes into a 24-bit number and sign-extend to 32 bits
            int value = (data[2] << 16) | (data[1] << 8) | data[0];

            // Check if the 24-bit integer's sign bit is set; if so, extend the sign to the 32-bit integer
            if ((value & 0x800000) != 0)
            {
                value |= unchecked((int)0xFF000000); // Sign-extend to 32 bits by setting the upper 8 bits
            }
            return value;
        }

        static short Parse16(Span<byte> data)
        {
            if (data.Length < 2)
                throw new ArgumentException("Data span is too short to contain a signed 16-bit integer.");

            // Combine the bytes into a 16-bit number
            short value = (short)((data[1] << 8) | data[0]);

            return value;
        }

        static void InitGyro(I2cDevice device)
        {
            Span<byte> setupAccGyro = stackalloc byte[] { 0x10, 0x10, 0x10, 0x44, 0x10, 0x80 };
            device.Write(setupAccGyro);
        }

        static void ReadPressureTemp(I2cDevice device)
        {
            float p = 1.0f / 40960.0f;
            float t = 0.01f;

            // Byte to be sent to the device
            Span<byte> measurePressure = stackalloc byte[] { 0x11, 0x11 };
            Span<byte> From = stackalloc byte[] { 0x28 };
            // Buffer to hold the read data
            Span<byte> PressureTemp = stackalloc byte[5];

            // Send command to measure the values
            device.Write(measurePressure);

            //wait measurement to happen
            Thread.Sleep(10);

            // Read Data from the sensor
            device.WriteRead(From, PressureTemp);

            P = p * Parse24(PressureTemp.Slice(0, 3));
            TPressure = t * Parse16(PressureTemp.Slice(3, 2));
           
            //Console.WriteLine($"| {"Press",12} : {P,12:F3} | {"",12}   {"",12} | {"kPa ",6} |");
            //Console.WriteLine($"| {"Temp",12} : {T,12:F3} | {"",12}   {"",12} | {"°C  ",6} |");
        }

        static void ReadAccGyroTemp(I2cDevice device)
        {
            float g = 0.061f;
            float G = 8.75f;
            float t = 0.0039f;

            // Byte to be sent to the device
            Span<byte> From = stackalloc byte[1]; //{ 0x20 };

            Span<byte> Status = stackalloc byte[1];
            // Buffer to hold the read data
            Span<byte> TempGyroAcc = stackalloc byte[14];

            //wait measurement to happen
            From[0] = 0x1E;
            Status[0] = 0x00;
            do
                device.WriteRead(From, Status);
            while ((Status[0] & 0x01) == 0);

            // Read Data from the sensor
            From[0] = 0x20;
            device.WriteRead(From, TempGyroAcc);

            TGyro = t * Parse16(TempGyroAcc.Slice(0, 2));
           
            X = g * Parse16(TempGyroAcc.Slice(2, 2));
            Y = g * Parse16(TempGyroAcc.Slice(4, 2));
            Z = g * Parse16(TempGyroAcc.Slice(6, 2));
           
            XX = G * Parse16(TempGyroAcc.Slice(8, 2));
            YY = G * Parse16(TempGyroAcc.Slice(10, 2));
            ZZ = G * Parse16(TempGyroAcc.Slice(12, 2));
          
            //Console.WriteLine($"| {"Acc",12} : {X,12:F3} | {Y,12:F3} | {Z,12:F3} | {"mg  ",6} |");
            //Console.WriteLine($"| {"Gyro",12} : {XX,12:F3} | {YY,12:F3} | {ZZ,12:F3} | {"md/s ",6} |");
            //Console.WriteLine($"| {"Temp",12} : {(T + 25.0f),12:F3} | {"",12}   {"",12} | {"°C  ",6} |");
        }
    }
}

